#include <errno.h>
#include <libgen.h>
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include "parse_conf.h"

int main(int argc, char **argv) {
	int i;
	struct service_config sc;
	const char *p;
	char buff[LINE_MAX];
	p = basename(argv[0]);
	if(0 == strcmp(p, "s3c")) {
		argv++;
		argc--;
	}
	memset(&sc, 0, sizeof(sc));
	if(strchr(argv[0], '/') != NULL) {
		if(parse_file(argv[0], &sc) < 0) {
			if(*sc.parse_error)
				printf("%s: %s\n", argv[0], sc.parse_error);
			else
				perror(argv[0]);
			return 1;
		}
	} else {
		i = 0;
		snprintf(buff, sizeof(buff), "/lib/systemd/system/%s", argv[0]);
		if(parse_file(buff, &sc) < 0) {
			if(*sc.parse_error) {
				printf("%s: %s\n", argv[0], sc.parse_error);
				return 1;
			} else if(errno != ENOENT) {
				perror(argv[0]);
				return 1;
			}
		} else
			++i;
		snprintf(buff, sizeof(buff), "/etc/systemd/system/%s", argv[0]);
		if(parse_file(buff, &sc) < 0) {
			if(*sc.parse_error) {
				printf("%s: %s\n", argv[0], sc.parse_error);
				return 1;
			} else if(errno != ENOENT) {
				perror(argv[0]);
				return 1;
			}
		} else
			++i;
		snprintf(buff, sizeof(buff), "/run/systemd/system/%s", argv[0]);
		if(parse_file(buff, &sc) < 0) {
			if(*sc.parse_error) {
				printf("%s: %s\n", argv[0], sc.parse_error);
				return 1;
			} else if(errno != ENOENT) {
				perror(argv[0]);
				return 1;
			}
		} else
			++i;
		if(i == 0) {
			printf("%s: not found\n", argv[0]);
			return 1;
		}
	}
	if(argc < 2) {
		printf("%s: missing parameter\n", argv[0]);
		return 1;
	}
	if(0 == strcmp(argv[1], "check-syntax"))
		printf("%s: ok\n", argv[0]);
	else if(0 == strcmp(argv[1], "lsb-headers")) {
		puts("### BEGIN INIT INFO");
		if(sc.description[0] != '\0') {
			printf("# Short-Description: %s\n", sc.description);
			printf("# Description:       %s\n", sc.description);
		}
		if(sc.after[0] != '\0')
			/* TODO: translate names */
			printf("# Should-Start:      %s\n", sc.after);
		if(sc.before[0] != '\0')
			/* TODO: translate names */
			printf("# X-Start-Before:    %s\n", sc.before);
		puts("### END INIT INFO");
	} else if(0 == strcmp(argv[0], "start")) {
		if(sc.type == ST_ONESHOT)
			printf("TODO execute and wait for completion: %s\n",
					sc.exec_start);
		else
			printf("TODO execute in background: %s\n",
					sc.exec_start);
	} else {
		/* TODO: implement stop status restart force-reload */
		printf("%s: unknown action %s\n", argv[0], argv[1]);
		return 1;
	}
	return 0;
}
